import time

from redis.cluster import RedisCluster
from urllib3.exceptions import ConnectTimeoutError

import src.curl_client_lib
import src.druid_client_lib
import src.redis_client_lib
import src.site_service_client_lib
from src.config import Config
from src.druid_client_lib import DruidClient, DruidQueryError
from src.logger import get_logger
from src.redis_client_lib import RedisClientError

logger = get_logger(__name__)


def warm_latest_flow_cache(site_id: str, redis_client: RedisCluster, druid_client: DruidClient) -> None:
    logger.info("found the redis flow key to be empty, will attempt cache warming...")

    # query druid and get all data (limit 1M), then serialize data into proper format for redis and store it in redis
    logger.info("retrieving data from druid...")
    results = src.druid_client_lib.query_druid(druid_client)
    logger.info(f"got {len(results)} records from druid.")

    logger.info("storing site info in redis")
    src.redis_client_lib.store_site_info(redis_client, site_id)

    logger.info("chunking results and storing in redis...")
    src.redis_client_lib.chunk_redis_map_and_store(redis_client, results, site_id)
    logger.info("cache warming succeeded.")


def warm_latest_flow_cache_with_retry(site_id: str, redis_client: RedisCluster, druid_client: DruidClient) -> None:
    for i in range(Config.RETRY_COUNT):
        try:
            if src.redis_client_lib.key_is_not_empty(redis_client, hash_key):
                logger.info("determined that flow key is not empty, will skip cache warming.")
                return

            warm_latest_flow_cache(site_id, redis_client, druid_client)
            return
        except (DruidQueryError, RedisClientError, ConnectTimeoutError) as e:
            logger.warn(
                f"got error {e} when attempting cache warming on try {i+1} out of {Config.RETRY_COUNT}, waiting for {Config.RETRY_FREQUENCY_SECONDS} seconds before retrying"
            )
            time.sleep(Config.RETRY_FREQUENCY_SECONDS)


if __name__ == "__main__":
    # check if cache warming is necessary
    logger.info("starting redis cache warming script...")
    logger.warning("this script assumes a single-site deployment!")
    curl_client = src.curl_client_lib.retry_session(3)

    logger.info("getting site id...")
    site_id = src.site_service_client_lib.get_site_id_with_retry(curl_client)

    logger.info("creating redis client...")
    redis_client = src.redis_client_lib.create_redis_client()
    hash_key = src.redis_client_lib.format_site_data_key(site_id, Config.LATEST_FLOW_DATA_MAP_IDENTIFIER)

    logger.info("creating druid client...")
    druid_client = DruidClient(curl_client)

    warm_latest_flow_cache_with_retry(site_id, redis_client, druid_client)
