import json
import math
import uuid
from datetime import datetime, timedelta

import requests

from src.config import Config
from src.logger import get_logger

logger = get_logger(__name__)


class DruidQueryError(Exception):
    pass


class DruidClient:
    def __init__(self, session: requests.Session):
        self._curl_client = session

    def query_druid(self, query):
        url = f"{Config.DRUID_URL}/druid/v2"
        logger.debug(f"calling druid api at {url}")

        logger.info("querying for all druid flow data in the last 30 days")
        response = self._curl_client.post(url, json=query, allow_redirects=False)
        logger.debug(response.text)
        if response.status_code != requests.codes.ok:
            logger.warning(f"failed to query druid at {url} with status {response.status_code}:")
            raise DruidQueryError(f"failed to query druid at {url} with status {response.status_code}")

        result = json.loads(response.text)
        return result


def create_epoch_values(start_time: datetime, end_time: datetime) -> tuple[int, int]:
    return math.floor(start_time.timestamp()), math.floor(end_time.timestamp())


def create_query_context() -> str:
    query_context = Config.QUERY_ID_PREFIX + str(uuid.uuid4())
    return query_context


def create_time_interval() -> tuple[str, tuple[int, int]]:
    now = datetime.now()
    end_time = now.strftime(Config.INTERVAL_TEMPLATE)
    start_time = now - timedelta(days=30)

    # we round the start time down to the nearest day for the query
    start_time_rounded = now - timedelta(
        days=30,
        hours=int(now.hour),
        minutes=int(now.minute),
        seconds=int(now.second),
        microseconds=int(now.microsecond),
    )
    start_time_rounded = start_time_rounded.strftime(Config.INTERVAL_TEMPLATE)

    interval = f"{start_time_rounded}/{end_time}"
    return interval, create_epoch_values(start_time, now)


def query_druid(druid_client: DruidClient):
    with open(Config.DRUID_QUERY_TEMPLATE_FILEPATH, "r") as json_file:
        query = json.load(json_file)

    query["context"]["queryId"] = create_query_context()

    interval, (epoch_start, epoch_end) = create_time_interval()
    query["intervals"] = [interval]
    query["filter"]["lower"] = epoch_start
    query["filter"]["upper"] = epoch_end

    return druid_client.query_druid(query)
