import json
import os
from typing import Any

from redis.cluster import RedisCluster

from src.config import Config
from src.logger import get_logger

logger = get_logger(__name__)


class RedisClientError(Exception):
    pass


def format_site_data_key(site_id: str, key_id: str) -> str:
    return f"{{{site_id}}}:{key_id}"


def retrieve_redis_password() -> str:
    return os.environ[Config.REDIS_PASSWORD_ENV]


def create_redis_client() -> RedisCluster:
    return RedisCluster(
        host=Config.REDIS_HOST,
        port=Config.REDIS_PORT,
        password=retrieve_redis_password(),
        decode_responses=True,
        ssl=True,
        ssl_keyfile=os.path.join(Config.CERT_PATH, Config.KEY_CERT),
        ssl_certfile=os.path.join(Config.CERT_PATH, Config.PUBLIC_CERT),
        ssl_cert_reqs="required",
        ssl_ca_certs=os.path.join(Config.CERT_PATH, Config.CA_CERT),
    )  # type: ignore


def key_is_not_empty(redis_client: RedisCluster, key: str) -> bool:
    return redis_client.exists(key) > 0 and redis_client.hlen(key) > 0  # type: ignore


def store_site_info(redis_client: RedisCluster, site_id: str) -> None:
    site_info = {
        "@class": "com.vmware.nsx.pace.latestflowdata.LatestFlowSiteDataConfig",
        "siteId": site_id,
        "latestFlowDataMapName": format_site_data_key(site_id, Config.LATEST_FLOW_DATA_MAP_IDENTIFIER),
        "latestFlowDataSortedSetName": format_site_data_key(site_id, Config.LATEST_FLOW_DATA_SET_IDENTIFIER),
        "totalFlowMetricsMapName": format_site_data_key(site_id, Config.TOTALFLOW_MAP_IDENTIFIER),
        "totalFlowMetricsSortedSetName": format_site_data_key(site_id, Config.TOTALFLOW_SET_IDENTIFIER),
        "workloadDataMapName": format_site_data_key(site_id, Config.WORKLOAD_MAP_IDENTIFIER),
        "workloadDataSortedSetName": format_site_data_key(site_id, Config.WORKLOAD_SET_IDENTIFIER),
        "monitorMetricsMapName": format_site_data_key(site_id, Config.MONITOR_METRICS_IDENTIFIER),
    }

    site_info_hash = {site_id: json.dumps(site_info)}

    success = redis_client.hmset(Config.SITE_INFO_MAP_KEY, site_info_hash)  # type: ignore
    logger.debug(f"redis response when storing site info: {success}")
    if success is not True:
        logger.error("failed to set site info in redis")
        raise RedisClientError("failed to set site info in redis")


def store_data_in_redis(redis_client: RedisCluster, output_map, hash_key: str, sorted_set_key: str) -> None:
    success = redis_client.hmset(hash_key, output_map)
    logger.debug(f"redis response when storing data: {success}")

    if success is not True:
        logger.error("failed to set druid data in redis")
        raise RedisClientError("failed to set druid data in redis")

    for flow_key, event in output_map.items():
        # event stored as string, need to load into json object to retrieve lastEndTime
        dict_event = json.loads(event)
        last_end_time = dict_event["lastEndTime"]

        # they key used for the sorted set has escaped quotations, so need to format it as such via the f-string below
        redis_client.zadd(sorted_set_key, {f'"{flow_key}"': last_end_time})


def chunk_redis_map_and_store(
    redis_client: RedisCluster, results: list[dict[str, str | int | None]], site_id: str
) -> None:
    hash_key = format_site_data_key(site_id, Config.LATEST_FLOW_DATA_MAP_IDENTIFIER)
    sorted_set_key = format_site_data_key(site_id, Config.LATEST_FLOW_DATA_SET_IDENTIFIER)
    results_length = len(results)
    num_chunks = 1
    start = 0
    end = min(min(Config.MAX_FLOW_COUNT, Config.MSET_REDIS_DATA_BLOCK_SIZE), results_length)
    while start < results_length and start < Config.MAX_FLOW_COUNT:
        logger.info("Serializing chunk {} data into redis format".format(num_chunks))
        submap = serialize_output(results[start:end])
        logger.info("Storing chunk {} data in redis".format(num_chunks))
        store_data_in_redis(redis_client, submap, hash_key, sorted_set_key)
        start = end
        end = min(
            min(end + Config.MSET_REDIS_DATA_BLOCK_SIZE, Config.MAX_FLOW_COUNT),
            results_length,
        )
        num_chunks += 1


def serialize_output(results: list[dict[str, Any]]) -> dict[str, Any]:
    outputMap = {}
    for result in results:
        event = serialize_event(result["event"])
        flowKey = event["flowKey"]
        del event["flowKey"]
        outputMap[flowKey] = json.dumps(event)
    return outputMap


def serialize_event(event: dict[str, Any]) -> dict[str, Any]:
    event["srcComputeInfo"] = None
    event["dstComputeInfo"] = None
    event["numFwAllowed"] = None
    event["numFwBlocked"] = None
    event["numUnMicrosegmented"] = None
    event["numVds"] = None
    # needed to deserialize string into java class
    event["@class"] = "com.vmware.nsx.pace.latestflowdata.LatestFlow"

    event["lastEndTime"] = int(event["lastEndTime"])

    # src / dst group ids will be a string if only 1 value is returned, an array if 2+ are returned
    # check for it being a string value and turn it into an array if it is a string
    srcGroupIds = event["srcGroupIds"]
    dstGroupIds = event["dstGroupIds"]
    if isinstance(srcGroupIds, str):
        srcGroupIds = [srcGroupIds]
    if isinstance(dstGroupIds, str):
        dstGroupIds = [dstGroupIds]

    event["srcGroupIds"] = ["java.util.ArrayList", srcGroupIds]
    event["dstGroupIds"] = ["java.util.ArrayList", dstGroupIds]
    return event
