import json
import time

import requests

from src.config import Config
from src.logger import get_logger

logger = get_logger(__name__)

DEFAULT_SITE_ID = "default"


class SiteServiceClientError(Exception):
    pass


class MissingSiteError(Exception):
    pass


def get_site_id_with_retry(session: requests.Session) -> str:
    for i in range(Config.GET_SITE_ID_RETRY_COUNT):
        try:
            return get_site_id(session)
        except MissingSiteError:
            logger.warn(
                f"got no sites from site service API on try {i+1} out of {Config.GET_SITE_ID_RETRY_COUNT}, waiting for {Config.GET_SITE_ID_RETRY_FREQUENCY_SECONDS} seconds before retrying"
            )
            time.sleep(Config.GET_SITE_ID_RETRY_FREQUENCY_SECONDS)

    logger.error(
        f"got no site IDs from site service after {Config.GET_SITE_ID_RETRY_COUNT} retries, cannot perform cache warming without a site ID"
    )
    raise SiteServiceClientError("expected 1 site, got 0")


def get_site_id(session: requests.Session) -> str:
    # assumes only one site
    url = f"{Config.SITE_SERVICE_URL}/sites"
    logger.debug(f"calling site service api at {url}")

    response = session.get(url, allow_redirects=False)
    logger.debug(response.text)
    if response.status_code != requests.codes.ok:
        logger.warning(f"failed to call site service {url} with status {response.status_code}:")
        raise SiteServiceClientError(f"failed to call site service {url} with status {response.status_code}")

    result = json.loads(response.text)
    if "sites_info" not in result or len(result["sites_info"]) == 0:
        logger.error("got no site IDs from site service, cannot perform cache warming without a site ID")
        raise MissingSiteError("expected 1 site, got 0")

    sites_list = result["sites_info"]
    if len(sites_list) != 1:
        logger.error("site_service_lib only supports single site deployments!")
        raise SiteServiceClientError(f"expected 1 site, got {len(sites_list)}")

    site_id = sites_list[0]["site_id"]
    if site_id == DEFAULT_SITE_ID:
        logger.error(
            f"got default site ID {DEFAULT_SITE_ID} when warming cache, this is probably in error, a valid site ID is required"
        )
        raise SiteServiceClientError(f"got invalid site id, {DEFAULT_SITE_ID}")

    return site_id
